/******************************************************************************
 *                  QLOGIC LINUX SOFTWARE
 *
 * QLogic ISP2x00 IP network driver for Linux 2.4.x
 * Copyright (C) 2002 Qlogic Corporation
 * (www.qlogic.com)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 ******************************************************************************/

/****************************************************************************
              Please see revision.notes for revision history.
*****************************************************************************/

#ifndef _QLA_IP_H_
#define _QLA_IP_H_

#define MAX_SEND_PACKETS            32      /* Maximum number of send packets */
#define MAX_RECEIVE_BUFFERS         64      /* Maximum number of receive buffers */
#define MIN_RECEIVE_BUFFERS         8       /* Minimum number of receive buffers */
#define IP_BUFFER_QUEUE_DEPTH       (MAX_RECEIVE_BUFFERS+1)

/* Async notification types */
#define NOTIFY_EVENT_LINK_DOWN      1       /* Link went down */
#define NOTIFY_EVENT_LINK_UP        2       /* Link is back up */
#define NOTIFY_EVENT_RESET_DETECTED 3       /* Reset detected */

/* QLogic subroutine status definitions */
#define QL_STATUS_SUCCESS           0
#define QL_STATUS_ERROR             1
#define QL_STATUS_FATAL_ERROR       2
#define QL_STATUS_RESOURCE_ERROR    3
#define QL_STATUS_LOOP_ID_IN_USE    4
#define QL_STATUS_NO_DATA           5

/************************************************************************/
/*  Fibre IP header definitions.                                        */
/************************************************************************/

/* Network header definitions */

typedef struct _NETWORK_HEADER
{
    uint16_t        wDestinationNAA;        /* Destination NAA (0x0010) */
    uint16_t        wDestinationAddrHigh;   /* Destination address (high) */
    uint32_t        lDestinationAddrLow;    /* Destination address (low) */
    uint16_t        wSourceNAA;             /* Source NAA (0x0010) */
    uint16_t        wSourceAddrHigh;        /* Source address (high) */
    uint32_t        lSourceAddrLow;         /* Source address (low) */
} NETWORK_HEADER, *PNETWORK_HEADER;

#define NAA_IEEE_MAC_TYPE       0x0010      /* NAA code for IEEE MAC address */
#define BROADCAST_ADDRESS       0xFFFFFFFF
#define BROADCAST_ADDRESS_WORD  0xFFFF

/* SNAP header definitions */

typedef struct _SNAP_HEADER
{
    uint8_t         cDestinationSAP;        /* LLC destination service access point */
    uint8_t         cSourceSAP;             /* LLC source service access point */
    uint8_t         cControl;               /* LLC control */
    uint8_t         caSnapOUI[3];           /* SNAP OUI */
    uint16_t        wSnapPID;               /* SNAP PID */
} SNAP_HEADER, *PSNAP_HEADER;

#define LLC_SAP_IEEE_802DOT2    0xAA        /* LLC SAP code for IEEE 802.2 */
#define LLC_CONTROL             0x03        /* LLC control code */
#define SNAP_OUI                0x00        /* SNAP OUI code */
#define SNAP_PID_IP_TYPE        0x0008      /* IP header type */
#define SNAP_PID_ARP_TYPE       0x0608      /* ARP header type */

/* Packet header definitions */

typedef struct _PACKET_HEADER
{
    NETWORK_HEADER  sNetworkHeader;
    SNAP_HEADER     sSnapHeader;
} PACKET_HEADER, *PPACKET_HEADER;

/* ARP header definitions */

typedef struct _ARP_HEADER
{
    uint16_t        wHardwareType;
    uint16_t        wProtocolType;
    uint8_t         bHLen;
    uint8_t         bPLen;
    uint16_t        wOperation;
    uint8_t         acSenderHA[6];
    uint8_t         acSenderIp[4];
    uint8_t         acTargetHA[6];
    uint8_t         acTargetIp[4];
} ARP_HEADER, *PARP_HEADER;

#define ARP_HARDWARE_TYPE   0x0600
#define ARP_PROTOCAL_TYPE   0x0008
#define ARP_HLEN            0x06
#define ARP_PLEN            0x04
#define ARP_OPERATION       0x0100

/* ARP packet header definitions */

typedef struct _ARP_PACKET_HEADER
{
    NETWORK_HEADER  sNetworkHeader;
    SNAP_HEADER     sSnapHeader;
    ARP_HEADER      sArpHeader;
} ARP_PACKET_HEADER, *PARP_PACKET_HEADER;

/* IP header definitions */

typedef struct _IP_HEADER
{
    uint8_t         bVersionHLength;
    uint8_t         bServiceType;
    uint16_t        wTotalLength;
    uint16_t        wIdentification;
    uint16_t        wFlagsOffset;
    uint8_t         bTimeToLive;
    uint8_t         bProtocol;
    uint16_t        wHChecksum;
    uint8_t         acSourceIpAddress[4];
    uint8_t         acDestinationIpAddress[4];
    uint32_t        lIpOptions;
} IP_HEADER, *PIP_HEADER;

/* IP packet header definitions */

typedef struct _IP_PACKET_HEADER
{
    NETWORK_HEADER  sNetworkHeader;
    SNAP_HEADER     sSnapHeader;
    IP_HEADER       sIpHeader;
} IP_PACKET_HEADER, *PIP_PACKET_HEADER;

/* RISC IP receive buffer queue entry structure */

typedef struct  _IpBufferEntry
{
    uint32_t        lIpBufferAddrLow;
    uint32_t        lIpBufferAddrHigh;
    uint16_t        wIpBufferHandle;
    uint16_t        wReserved;
} IP_BUFFER_ENTRY, *PIP_BUFFER_ENTRY;

/* Receive buffer control block definitions */

typedef struct _BufferCb
{
    struct _BufferCb *pNextBufferCB;        /* Pointer to next buffer control block */
    struct sk_buff  *skb;                   /* Pointer to IP packet */
    uint8_t         *pBuffer;               /* Pointer to data buffer */
    uint32_t        physicalAddressLow;     /* Buffer physical address */
    uint32_t        physicalAddressHigh;    /* Buffer physical address */
    uint16_t        wBufferHandle;          /* ISP buffer handle */
    uint16_t        wBufferCount;           /* Number of linked buffers for packet */
    uint32_t        lBufferSize;            /* Size of received data in buffer */
    uint32_t        lPacketSize;            /* Size of entire packet received */
    uint32_t        lFlags;                 /* Buffer control block flags */
} BUFFER_CB,*PBUFFER_CB;

#define BCB_FLAGS_RISC_OWNS_BUFFER      0x00000001

/* Send control block definitions */

typedef struct _SendCb
{
    void            *qdev;                  /* Pointer to device private structure */
    struct sk_buff  *skb;                   /* Pointer to IP packet to send */
    uint32_t        physicalAddressLow;     /* Header physical address */
    uint32_t        physicalAddressHigh;    /* Header physical address */
    PACKET_HEADER   header;                 /* Network and SNAP headers */
} SEND_CB,*PSEND_CB;

/************************************************************************/
/*  Definitions for Backdoor Inquiry.                                   */
/************************************************************************/

typedef struct  _BdInquiryData
{
    uint16_t        wLength;                /* Length of structure */
    uint16_t        wVersion;               /* Version number of structure */
    uint32_t        lOptions;               /* Export miniport supported options */
    void            *pScsiDriverHa;         /* Export driver ha pointer */
    void            *pReceiveBufferQueue;   /* Export receive buffer queue pointer */
    uint16_t        wReceiveBufferQueueSize;/* Export receive buffer queue size */
    uint16_t        wUnused1;
    uint8_t         acPortName[8];          /* Export adapter port name */
    uint32_t        alUnused[9];
} BD_INQUIRY_DATA, *PBD_INQUIRY_DATA;

#define BDI_LENGTH  sizeof(BD_INQUIRY_DATA) /* Length of BD_INQUIRY_DATA */
#define BDI_VERSION 1                       /* Version of BD_INQUIRY_DATA */

/* BD_INQUIRY options */

#define BDI_IP_SUPPORT      0x00000001      /* IP driver supported */

/************************************************************************/
/*  Definitions for Backdoor Enable.                                    */
/************************************************************************/

typedef struct  _BdEnableData
{
    uint16_t        wLength;                /* Length of structure */
    uint16_t        wVersion;               /* Version number of structure */
    uint32_t        lOptions;               /* Import driver supported options */
    void            *pNotifyRoutine;        /* Import notification routine */
    void            *pNotifyContext;        /* Import notification parameter */
    uint16_t        wReceiveBufferCount;    /* Import buffer count */
    uint16_t        wHeaderSize;            /* Import split header size */
    uint32_t        lReceiveBufferSize;     /* Import buffer size */
    uint32_t        lMaximumTransferSize;   /* Import maximum transfer size */
    void            *pReturnReceivePacketsRoutine; /* Import return packets routine */
    void            *pReturnReceivePacketsContext; /* Import return packets parameter */
    void            *pReceiveBufferCBs;      /* Import receive buffer control blocks */
    void            *pSendCompletionRoutine; /* Import send completion routine */
    uint32_t        alUnused[5];
} BD_ENABLE_DATA, *PBD_ENABLE_DATA;

#define BDE_LENGTH  sizeof(BD_ENABLE_DATA)  /* Length of BD_ENABLE_DATA */
#define BDE_VERSION 1                       /* Version of BD_ENABLE_DATA */

/* BD_ENABLE options */

#define BDE_NOTIFY_ROUTINE  0x00000001      /* Notify routine imported */

/* Callback routines from SCSI driver to IP driver */
typedef void (*PNOTIFY_CALLBACK) (void *context, uint32_t type);
typedef void (*PRECEIVE_PACKETS_CALLBACK) (void *context, BUFFER_CB *pBufferCB);
typedef void (*PSEND_COMPLETION_CALLBACK) (void *pSendCB);



/*
 * Definitions for IP support 
 */
#define QLA2100_DEVICE_ID       0x2100
#define LOOP_ID_MASK            0x00FF
#define PLE_NOT_SCSI_DEVICE     0x8000  /* Upper bit of loop ID set if not SCSI */

/* Firmware IP initialization control block definitions */
typedef struct  _IPInitBlock
{
    uint8_t     cVersion;
    uint8_t	    cReserved1;
    uint16_t	wIpFirmwareOptions;
    uint16_t	wIpHeaderSize;
    uint16_t	wIpMTUSize;
    uint16_t	wIpBufferSize;
    uint16_t	wReserved2;
    uint16_t	wReserved3;
    uint16_t	wReserved4;
    uint16_t	wReserved5;
    uint16_t	wIpQueueSize;
    uint16_t	wIpLowWaterMark;
    uint16_t	wIpQueueAddr[4];
    uint16_t	wIpQueueInPointer;
    uint16_t	wIpFastPostCount;
    uint16_t	wIpBufferContainerCount;
    uint16_t	wIpIocbResourceAllocation;
} IP_INIT_BLOCK, *PIP_INIT_BLOCK;

#define IPIB_VERSION                    0x01
#define IPIB_LOW_WATER_MARK             0
#define IPIB_FAST_POST_COUNT            4
#define IPIB_BUFFER_CONTAINER_COUNT     64
#define IPIB_IOCB_RESERVE_COUNT         250

/* IP firmware options */
#define IPIB_OPTION_64BIT_ADDRESSING            0x0001
#define IPIB_OPTION_NO_BROADCAST_FASTPOST       0x0002
#define IPIB_OPTION_OUT_OF_BUFFERS_EVENT        0x0004

/* IP device data structure */
typedef struct _IpDeviceBlock
{
    uint32_t    lDeviceFlags;       
    uint16_t    wLoopId;            
    uint16_t    wDeviceIndex;
    uint8_t     acWorldWideName[8]; 
    uint8_t     acPortId[3];        
    uint8_t     cUnused2;
    struct _IpDeviceBlock *pNextIpDevice;      
    struct _IpDeviceBlock *pLastIpDevice;      
    uint32_t    lUnused3;
} IP_DEVICE_BLOCK, *PIP_DEVICE_BLOCK;

#define IP_DEV_FLAG_PRESENT         0x00000001 
#define IP_DEV_FLAG_PUBLIC_DEVICE   0x00000002

#define PUBLIC_LOOP_DEVICE          ((uint16_t)-1)

/* Maximum number of IP connections */
#define QLLAN_MAX_IP_DEVICES        64

/* IP IOCB types */
#define ET_IP_COMMAND_32            0x13
#define ET_IP_COMMAND_64            0x1B
#define ET_IP_RECEIVE               0x23
#define ET_IP_RECEIVE_CONTINUATION  0x2B
#define ET_MAILBOX_COMMAND          0x39

/* Data segment descriptor structures */
typedef struct
{
    uint32_t    lBase;
    uint32_t    lCount;
} DATA_SEG;

typedef struct
{
    uint32_t    lBase[2];
    uint32_t    lCount;
} DATA_SEG_A64;

/* IP Command IOCB structure */
typedef struct  _IpCommandEntry
{
    uint32_t    lHeader;
    uint32_t    lHandle;
    uint8_t     cReserved1;
    uint8_t     cLoopId;
    uint16_t    wCompletionStatus;
    uint16_t    wControlFlags;
    uint16_t    wReserved2;
    uint16_t    wTimeout;
    uint16_t    wSegmentCount;
    uint16_t    wServiceClass;
    uint16_t    wReserved3[7];
    uint32_t    lTotalByteCount;
    union
    {
        DATA_SEG        asDataSeg[3];
        DATA_SEG_A64    asDataSeg64[2];
    } ds;
} IP_COMMAND_ENTRY, *PIP_COMMAND_ENTRY;

/* IP Receive IOCB structure */
#define IP_RECEIVE_ENTRY_MAX_HANDLES    24
typedef struct  _IpReceiveEntry
{
    uint8_t     cEntryType;
    uint8_t     cEntryCount;
    uint8_t     cSegmentCount;
    uint8_t     cEntryStatus;
    uint16_t    wS_IDLow;
    uint8_t     cS_IDHigh;
    uint8_t     cReserved1;
    uint8_t     cReserved2;
    uint8_t     cLoopId;
    uint16_t    wCompletionStatus;
    uint16_t    wServiceClass;
    uint16_t    wSequenseLength;
    uint16_t    waBufferHandle[IP_RECEIVE_ENTRY_MAX_HANDLES];
} IP_RECEIVE_ENTRY, *PIP_RECEIVE_ENTRY;

/* IP Receive IOCB completion status bits */
#define IP_REC_STATUS_SPLIT_BUFFER              0x0001

/* Mailbox command IOCB structure */
typedef struct  _MbxIocb
{
    uint32_t    Header;
    uint32_t    Handle;
    uint8_t     cReserved1;
    uint8_t     LoopId;
    uint16_t    Status;
    uint16_t    StateFlags;
    uint16_t    StatusFlags;
    uint16_t    SysDef2[4];
    uint16_t    Mailbox0;
    uint16_t    Mailbox1;
    uint16_t    Mailbox2;
    uint16_t    Mailbox3;
    uint16_t    Mailbox6;
    uint16_t    Mailbox7;
    uint32_t    Reserved[7];
} MBX_IOCB, *PMBX_IOCB;

/* Use SystemDefined1 in IOCB header for source type */
#define SOURCE_SCSI           0x00
#define SOURCE_IP             0x01
#define SOURCE_VI             0x02
#define SOURCE_SCTP           0x03
#define SOURCE_MP             0x04
#define SOURCE_MPIOCTL        0x05

/* IP mailbox commands */
#define MBC_INITIALIZE_IP               0x0077
#define MBC_DISABLE_IP                  0x0079

/* Fabric login mailbox command option bits */
#define MBC_NO_PLOGI_IF_LOGGED_IN       0x01
#define MBC_NO_PROCESS_LOGIN            0x02

/* Mailbox command completion status */
#define MBS_PORT_ID_IN_USE              0x4007
#define MBS_LOOP_ID_IN_USE              0x4008
#define MBS_ALL_LOOP_IDS_IN_USE         0x4009
#define MBS_NAME_SERVER_NOT_LOGGED_IN   0x400A

/* IP async events */
#define MBA_IP_TRANSMIT_COMPLETE        0x8022
#define MBA_IP_RECEIVE_COMPLETE         0x8023
#define MBA_IP_BROADCAST_RECEIVED       0x8024
#define MBA_IP_RECEIVE_BUFFERS_LOW      0x8025
#define MBA_IP_OUT_OF_BUFFERS           0x8026
#define MBA_IP_RECEIVE_COMPLETE_SPLIT   0x8027

/* IP fast post completions for 2300 */
#define RHS_IP_SEND_COMPLETE            0x18
#define RHS_IP_RECV_COMPLETE            0x19
#define RHS_IP_RECV_DA_COMPLETE         0x1B

#endif /* _QLA_IP_H_ */
